@@ -330,7 +330,9 @@ static void print_odp_trans_caps(uint32_t trans)
 					    IBV_ODP_SUPPORT_WRITE |
 					    IBV_ODP_SUPPORT_READ |
 					    IBV_ODP_SUPPORT_ATOMIC |
-					    IBV_ODP_SUPPORT_SRQ_RECV);
+					    IBV_ODP_SUPPORT_SRQ_RECV |
+					    IBV_ODP_SUPPORT_FLUSH |
+					    IBV_ODP_SUPPORT_ATOMIC_WRITE);
 
 	if (!trans) {
 		printf("\t\t\t\t\tNO SUPPORT\n");
@@ -347,6 +349,10 @@ static void print_odp_trans_caps(uint32_t trans)
 			printf("\t\t\t\t\tSUPPORT_ATOMIC\n");
 		if (trans & IBV_ODP_SUPPORT_SRQ_RECV)
 			printf("\t\t\t\t\tSUPPORT_SRQ\n");
+		if (trans & IBV_ODP_SUPPORT_FLUSH)
+			printf("\t\t\t\t\tSUPPORT_FLUSH\n");
+		if (trans & IBV_ODP_SUPPORT_ATOMIC_WRITE)
+			printf("\t\t\t\t\tSUPPORT_ATOMIC_WRITE\n");
 		if (trans & unknown_transport_caps)
 			printf("\t\t\t\t\tUnknown flags: 0x%" PRIX32 "\n",
 			       trans & unknown_transport_caps);
