@@ -395,6 +395,23 @@ void ibv_unimport_dm(struct ibv_dm *dm)
 	get_ops(dm->context)->unimport_dm(dm);
 }
 
+/**
+ * ibv_alloc_dmah - Allocate a dma handle
+ */
+struct ibv_dmah *ibv_alloc_dmah(struct ibv_context *context,
+				struct ibv_dmah_init_attr *attr)
+{
+	return get_ops(context)->alloc_dmah(context, attr);
+}
+
+/**
+ * ibv_dealloc_dmah - Free a dma handle
+ */
+int ibv_dealloc_dmah(struct ibv_dmah *dmah)
+{
+	return get_ops(dmah->context)->dealloc_dmah(dmah);
+}
+
 struct ibv_mr *ibv_reg_dmabuf_mr(struct ibv_pd *pd, uint64_t offset,
 				 size_t length, uint64_t iova, int fd,
 				 int access)
@@ -413,6 +430,41 @@ struct ibv_mr *ibv_reg_dmabuf_mr(struct ibv_pd *pd, uint64_t offset,
 	return mr;
 }
 
+/* Note: mr_init_attr may be modified during this call */
+struct ibv_mr *ibv_reg_mr_ex(struct ibv_pd *pd, struct ibv_mr_init_attr *mr_init_attr)
+{
+	struct verbs_device *device = verbs_get_device(pd->context->device);
+	struct ibv_mr *mr;
+	int in_access = mr_init_attr->access;
+	bool need_fork = !((mr_init_attr->access & IBV_ACCESS_ON_DEMAND) ||
+			   (mr_init_attr->comp_mask & IBV_REG_MR_MASK_FD));
+
+	if (need_fork && ibv_dontfork_range(mr_init_attr->addr, mr_init_attr->length))
+		return NULL;
+
+	if (!(device->core_support & IB_UVERBS_CORE_SUPPORT_OPTIONAL_MR_ACCESS))
+		mr_init_attr->access &= ~IBV_ACCESS_OPTIONAL_RANGE;
+
+	mr = get_ops(pd->context)->reg_mr_ex(pd, mr_init_attr);
+	if (mr) {
+		mr->context = pd->context;
+		mr->length = mr_init_attr->length;
+		mr->pd = pd;
+		if (mr_init_attr->comp_mask & IBV_REG_MR_MASK_ADDR)
+			mr->addr = mr_init_attr->addr;
+		else
+			/* Follows ibv_reg_dmabuf_mr logic */
+			mr->addr = (void *)(uintptr_t) mr_init_attr->fd_offset;
+	} else {
+		if (need_fork)
+			ibv_dofork_range(mr_init_attr->addr, mr_init_attr->length);
+	}
+
+	/* restore the input access flags */
+	mr_init_attr->access = in_access;
+	return mr;
+}
+
 LATEST_SYMVER_FUNC(ibv_rereg_mr, 1_1, "IBVERBS_1.1",
 		   int,
 		   struct ibv_mr *mr, int flags,
