@@ -2,7 +2,8 @@ from tests.base import RCResources, UDResources, XRCResources, RDMATestCase, \
     PyverbsAPITestCase
 from pyverbs.pyverbs_error import PyverbsRDMAError
 from pyverbs.cq import CqInitAttrEx, CQEX
-import pyverbs.enums as e
+from pyverbs.libibverbs_enums import ibv_access_flags, ibv_raw_packet_caps, ibv_create_cq_wc_flags, \
+    ibv_cq_init_attr_mask, ibv_create_cq_attr_flags, IBV_WC_STANDARD_FLAGS
 from pyverbs.mr import MR
 import tests.utils as u
 import unittest
@@ -16,7 +17,7 @@ def create_ex_cq(res):
     between devices.
     :param res: An instance of TrafficResources
     """
-    wc_flags = e.IBV_WC_STANDARD_FLAGS
+    wc_flags = IBV_WC_STANDARD_FLAGS
     cia = CqInitAttrEx(cqe=2000, wc_flags=wc_flags)
     try:
         res.cq = CQEX(res.ctx, cia)
@@ -31,7 +32,7 @@ class CqExUD(UDResources):
 
     def create_mr(self):
         self.mr = MR(self.pd, self.msg_size + self.GRH_SIZE,
-                     e.IBV_ACCESS_LOCAL_WRITE)
+                     ibv_access_flags.IBV_ACCESS_LOCAL_WRITE)
 
 
 class CqExRC(RCResources):
@@ -79,11 +80,11 @@ class CQEXAPITest(PyverbsAPITestCase):
         Test ibv_create_cq_ex()
         """
         cq_init_attrs_ex = CqInitAttrEx(cqe=10, wc_flags=0, comp_mask=0, flags=0)
-        if self.attr_ex.raw_packet_caps & e.IBV_RAW_PACKET_CAP_CVLAN_STRIPPING:
-            cq_init_attrs_ex.wc_flags = e.IBV_WC_EX_WITH_CVLAN
+        if self.attr_ex.raw_packet_caps & ibv_raw_packet_caps.IBV_RAW_PACKET_CAP_CVLAN_STRIPPING:
+            cq_init_attrs_ex.wc_flags = ibv_create_cq_wc_flags.IBV_WC_EX_WITH_CVLAN
             CQEX(self.ctx, cq_init_attrs_ex)
 
-        for flag in list(e.ibv_create_cq_wc_flags):
+        for flag in list(ibv_create_cq_wc_flags):
             cq_init_attrs_ex.wc_flags = flag
             try:
                 cq_ex = CQEX(self.ctx, cq_init_attrs_ex)
@@ -93,8 +94,8 @@ class CQEXAPITest(PyverbsAPITestCase):
                     raise ex
 
         cq_init_attrs_ex.wc_flags = 0
-        cq_init_attrs_ex.comp_mask = e.IBV_CQ_INIT_ATTR_MASK_FLAGS
-        attr_flags = list(e.ibv_create_cq_attr_flags)
+        cq_init_attrs_ex.comp_mask = ibv_cq_init_attr_mask.IBV_CQ_INIT_ATTR_MASK_FLAGS
+        attr_flags = list(ibv_create_cq_attr_flags)
         for flag in attr_flags:
             cq_init_attrs_ex.flags = flag
             try:
