@@ -12,12 +12,17 @@ from pyverbs.providers.mlx5.mlx5dv import Mlx5Context, Mlx5DVContextAttr, \
     Mlx5DVQPInitAttr, Mlx5QP
 from pyverbs.pyverbs_error import PyverbsRDMAError, PyverbsUserError
 from tests.mlx5_base import Mlx5RDMATestCase, Mlx5PyverbsAPITestCase
-import pyverbs.providers.mlx5.mlx5_enums as dve
+from pyverbs.providers.mlx5.mlx5_enums import mlx5dv_crypto_engines_caps, mlx5dv_crypto_wrapped_import_method_caps, \
+    mlx5dv_crypto_caps_flags, mlx5dv_mkey_init_attr_flags, mlx5dv_qp_init_attr_mask, mlx5dv_crypto_key_size, \
+    mlx5dv_crypto_key_purpose, mlx5dv_crypto_login_state, mlx5dv_dek_init_attr_mask, mlx5dv_signature_crypto_order, \
+    mlx5dv_crypto_standard, mlx5dv_block_size, mlx5dv_sig_crc_type, mlx5dv_sig_type, mlx5dv_sig_mask, \
+    mlx5dv_qp_create_send_ops_flags
 from pyverbs.wr import SGE, SendWR, RecvWR
 from pyverbs.qp import QPInitAttrEx, QPCap
 from tests.base import RCResources
 from pyverbs.pd import PD
-import pyverbs.enums as e
+from pyverbs.libibverbs_enums import ibv_qp_create_send_ops_flags, ibv_qp_init_attr_mask, ibv_qp_type, \
+    ibv_send_flags, ibv_access_flags, ibv_wr_opcode
 import tests.utils as u
 
 DEK_OPAQUE = b'dek'
@@ -64,9 +69,9 @@ def check_crypto_caps(dev_name, is_wrapped_dek_mode, multi_block_support=False):
     failed_selftests = crypto_caps['failed_selftests']
     if failed_selftests:
         raise unittest.SkipTest(f'The device crypto selftest failed ({failed_selftests})')
-    single_block_cap = dve.MLX5DV_CRYPTO_ENGINES_CAP_AES_XTS_SINGLE_BLOCK \
+    single_block_cap = mlx5dv_crypto_engines_caps.MLX5DV_CRYPTO_ENGINES_CAP_AES_XTS_SINGLE_BLOCK \
                        & crypto_caps['crypto_engines']
-    multi_block_cap = dve.MLX5DV_CRYPTO_ENGINES_CAP_AES_XTS_MULTI_BLOCK \
+    multi_block_cap = mlx5dv_crypto_engines_caps.MLX5DV_CRYPTO_ENGINES_CAP_AES_XTS_MULTI_BLOCK \
                       & crypto_caps['crypto_engines']
     if not single_block_cap and not multi_block_cap:
         raise unittest.SkipTest('The device crypto engines does not support AES')
@@ -77,11 +82,11 @@ def check_crypto_caps(dev_name, is_wrapped_dek_mode, multi_block_support=False):
               'The device crypto engines do not support single block but support multi blocks'
     dev_wrapped_import_method = crypto_caps['wrapped_import_method']
     if is_wrapped_dek_mode:
-        if not dve.MLX5DV_CRYPTO_WRAPPED_IMPORT_METHOD_CAP_AES_XTS & dev_wrapped_import_method or \
-                not dve.MLX5DV_CRYPTO_CAPS_WRAPPED_CRYPTO_OPERATIONAL & crypto_caps['flags']:
+        if not mlx5dv_crypto_wrapped_import_method_caps.MLX5DV_CRYPTO_WRAPPED_IMPORT_METHOD_CAP_AES_XTS & dev_wrapped_import_method or \
+                not mlx5dv_crypto_caps_flags.MLX5DV_CRYPTO_CAPS_WRAPPED_CRYPTO_OPERATIONAL & crypto_caps['flags']:
             raise unittest.SkipTest('The device does not support wrapped DEK')
-    elif dve.MLX5DV_CRYPTO_WRAPPED_IMPORT_METHOD_CAP_AES_XTS & dev_wrapped_import_method and \
-                dve.MLX5DV_CRYPTO_CAPS_WRAPPED_CRYPTO_OPERATIONAL & crypto_caps['flags']:
+    elif mlx5dv_crypto_wrapped_import_method_caps.MLX5DV_CRYPTO_WRAPPED_IMPORT_METHOD_CAP_AES_XTS & dev_wrapped_import_method and \
+                mlx5dv_crypto_caps_flags.MLX5DV_CRYPTO_CAPS_WRAPPED_CRYPTO_OPERATIONAL & crypto_caps['flags']:
         raise unittest.SkipTest('The device does not support plaintext DEK')
 
 
@@ -123,12 +128,12 @@ def requires_crypto_support(is_wrapped_dek_mode, multi_block_support=False):
 
 class Mlx5CryptoResources(RCResources):
     def __init__(self, dev_name, ib_port, gid_index, dv_send_ops_flags=0,
-                 mkey_create_flags=dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT,
+                 mkey_create_flags=mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT,
                  msg_size=1024):
         self.dv_send_ops_flags = dv_send_ops_flags
         self.mkey_create_flags = mkey_create_flags
         self.max_inline_data = 512
-        self.send_ops_flags = e.IBV_QP_EX_WITH_SEND
+        self.send_ops_flags = ibv_qp_create_send_ops_flags.IBV_QP_EX_WITH_SEND
         super().__init__(dev_name, ib_port, gid_index, msg_size=msg_size)
         self.create_mkeys()
 
@@ -146,16 +151,16 @@ class Mlx5CryptoResources(RCResources):
                      max_inline_data=self.max_inline_data)
 
     def create_qp_init_attr(self):
-        comp_mask = e.IBV_QP_INIT_ATTR_PD | e.IBV_QP_INIT_ATTR_SEND_OPS_FLAGS
+        comp_mask = ibv_qp_init_attr_mask.IBV_QP_INIT_ATTR_PD | ibv_qp_init_attr_mask.IBV_QP_INIT_ATTR_SEND_OPS_FLAGS
         return QPInitAttrEx(cap=self.create_qp_cap(), pd=self.pd, scq=self.cq,
-                            rcq=self.cq, qp_type=e.IBV_QPT_RC,
+                            rcq=self.cq, qp_type=ibv_qp_type.IBV_QPT_RC,
                             send_ops_flags=self.send_ops_flags,
                             comp_mask=comp_mask)
 
     def create_qps(self):
         try:
             qp_init_attr = self.create_qp_init_attr()
-            comp_mask = dve.MLX5DV_QP_INIT_ATTR_MASK_SEND_OPS_FLAGS
+            comp_mask = mlx5dv_qp_init_attr_mask.MLX5DV_QP_INIT_ATTR_MASK_SEND_OPS_FLAGS
             attr = Mlx5DVQPInitAttr(comp_mask=comp_mask,
                                     send_ops_flags=self.dv_send_ops_flags)
             qp = Mlx5QP(self.ctx, qp_init_attr, attr)
@@ -217,16 +222,16 @@ class Mlx5CryptoAPITest(Mlx5PyverbsAPITestCase):
             key = struct.pack('!5Q', *self.crypto_details['wrapped_key'])
             self.dek_init_attr = \
                 Mlx5DEKInitAttr(self.pd, key=key,
-                                key_size=dve.MLX5DV_CRYPTO_KEY_SIZE_128,
-                                key_purpose=dve.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS,
+                                key_size=mlx5dv_crypto_key_size.MLX5DV_CRYPTO_KEY_SIZE_128,
+                                key_purpose=mlx5dv_crypto_key_purpose.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS,
                                 opaque=DEK_OPAQUE)
             self.verify_create_dek_out_of_login_session()
-            self.verify_login_state(dve.MLX5DV_CRYPTO_LOGIN_STATE_NO_LOGIN)
+            self.verify_login_state(mlx5dv_crypto_login_state.MLX5DV_CRYPTO_LOGIN_STATE_NO_LOGIN)
 
             # Login to crypto session
             self.login_attr = Mlx5CryptoLoginAttr(cred_bytes)
             Mlx5Context.crypto_login(self.ctx, self.login_attr)
-            self.verify_login_state(dve.MLX5DV_CRYPTO_LOGIN_STATE_VALID)
+            self.verify_login_state(mlx5dv_crypto_login_state.MLX5DV_CRYPTO_LOGIN_STATE_VALID)
             self.verify_login_twice()
             self.dek = Mlx5DEK(self.ctx, self.dek_init_attr)
             self.verify_dek_opaque()
@@ -234,7 +239,7 @@ class Mlx5CryptoAPITest(Mlx5PyverbsAPITestCase):
 
             # Logout from crypto session
             Mlx5Context.crypto_logout(self.ctx)
-            self.verify_login_state(dve.MLX5DV_CRYPTO_LOGIN_STATE_NO_LOGIN)
+            self.verify_login_state(mlx5dv_crypto_login_state.MLX5DV_CRYPTO_LOGIN_STATE_NO_LOGIN)
         except PyverbsRDMAError as ex:
             print(ex)
             if ex.error_code == errno.EOPNOTSUPP:
@@ -253,20 +258,7 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         self.validate_data = False
         self.is_multi_block = False
         self.msg_size = 1024
-        self.key_size = dve.MLX5DV_CRYPTO_KEY_SIZE_128
-
-    def create_players(self, resource, **resource_arg):
-        """
-        Init Mlx5CryptoTest test resources.
-        :param resource: The RDMA resources to use.
-        :param resource_arg: Dict of args that specify the resource specific
-                             attributes.
-        :return: None
-        """
-        self.client = resource(**self.dev_info, **resource_arg)
-        self.server = resource(**self.dev_info, **resource_arg)
-        self.client.pre_run(self.server.psns, self.server.qps_num)
-        self.server.pre_run(self.client.psns, self.client.qps_num)
+        self.key_size = mlx5dv_crypto_key_size.MLX5DV_CRYPTO_KEY_SIZE_128
 
     def create_client_dek(self):
         """
@@ -284,7 +276,7 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         cred_bytes = struct.pack('!6Q', *self.crypto_details['credential'])
         log_attr = Mlx5CryptoExtLoginAttr(cred_bytes, 48)
         crypto_login_obj = Mlx5CryptoLogin(self.client.ctx, log_attr)
-        comp_mask=dve.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN
+        comp_mask=mlx5dv_dek_init_attr_mask.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN
         self._create_wrapped_dek(comp_mask, crypto_login_obj)
 
     def _create_wrapped_dek(self, comp_mask=0, crypto_login_obj=None):
@@ -292,18 +284,18 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         Create wrapped DEK using the client resources.
         """
         key = struct.pack('!5Q', *self.crypto_details['wrapped_key'])
-        if self.key_size == dve.MLX5DV_CRYPTO_KEY_SIZE_256:
+        if self.key_size == mlx5dv_crypto_key_size.MLX5DV_CRYPTO_KEY_SIZE_256:
             key = struct.pack('!9Q', *self.crypto_details['wrapped_256_bits_key'])
 
-        if comp_mask == dve.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN:
+        if comp_mask == mlx5dv_dek_init_attr_mask.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN:
             self.dek_attr = Mlx5DEKInitAttr(self.client.pd, key=key,
                                             key_size=self.key_size,
-                                            key_purpose=dve.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS,
+                                            key_purpose=mlx5dv_crypto_key_purpose.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS,
                                             comp_mask=comp_mask, crypto_login=crypto_login_obj)
         else:
             self.dek_attr = Mlx5DEKInitAttr(self.client.pd, key=key,
                                             key_size=self.key_size,
-                                            key_purpose=dve.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS)
+                                            key_purpose=mlx5dv_crypto_key_purpose.MLX5DV_CRYPTO_KEY_PURPOSE_AES_XTS)
 
         self.dek = Mlx5DEK(self.client.ctx, self.dek_attr)
 
@@ -312,11 +304,11 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         Create DEK using the client resources.
         """
         key = struct.pack('8I',*self.crypto_details['plaintext_key'])
-        if self.key_size == dve.MLX5DV_CRYPTO_KEY_SIZE_256:
+        if self.key_size == mlx5dv_crypto_key_size.MLX5DV_CRYPTO_KEY_SIZE_256:
             key = struct.pack('16I', *self.crypto_details['plaintext_256_bits_key'])
         self.dek_attr = Mlx5DEKInitAttr(self.client.pd,key=key,
                                         key_size=self.key_size,
-                                        comp_mask=dve.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN,
+                                        comp_mask=mlx5dv_dek_init_attr_mask.MLX5DV_DEK_INIT_ATTR_CRYPTO_LOGIN,
                                         crypto_login=None)
         self.dek = Mlx5DEK(self.client.ctx, self.dek_attr)
 
@@ -328,21 +320,21 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         num_of_configuration = 4 if signature else 3
         for mkey in [self.client.wire_enc_mkey, self.client.mem_enc_mkey]:
             self.client.qp.wr_start()
-            self.client.qp.wr_flags = e.IBV_SEND_SIGNALED | e.IBV_SEND_INLINE
+            self.client.qp.wr_flags = ibv_send_flags.IBV_SEND_SIGNALED | ibv_send_flags.IBV_SEND_INLINE
             offset = 0 if mkey == self.client.wire_enc_mkey else self.client.msg_size/2
             sge = SGE(self.client.mr.buf + offset, self.client.msg_size/2, self.client.mr.lkey)
             self.client.qp.wr_mkey_configure(mkey, num_of_configuration, Mlx5MkeyConfAttr())
-            self.client.qp.wr_set_mkey_access_flags(e.IBV_ACCESS_LOCAL_WRITE)
+            self.client.qp.wr_set_mkey_access_flags(ibv_access_flags.IBV_ACCESS_LOCAL_WRITE)
             self.client.qp.wr_set_mkey_layout_list([sge])
             if signature:
                 self.configure_mkey_signature()
             initial_tweak = struct.pack('!2Q', int(0), int(0))
             encrypt_on_tx = mkey == self.client.wire_enc_mkey
-            sign_crypto_order = dve.MLX5DV_SIGNATURE_CRYPTO_ORDER_SIGNATURE_BEFORE_CRYPTO_ON_TX
-            crypto_attr = Mlx5CryptoAttr(crypto_standard=dve.MLX5DV_CRYPTO_STANDARD_AES_XTS,
+            sign_crypto_order = mlx5dv_signature_crypto_order.MLX5DV_SIGNATURE_CRYPTO_ORDER_SIGNATURE_BEFORE_CRYPTO_ON_TX
+            crypto_attr = Mlx5CryptoAttr(crypto_standard=mlx5dv_crypto_standard.MLX5DV_CRYPTO_STANDARD_AES_XTS,
                                          encrypt_on_tx=encrypt_on_tx,
                                          signature_crypto_order=sign_crypto_order,
-                                         data_unit_size=dve.MLX5DV_BLOCK_SIZE_512,
+                                         data_unit_size=mlx5dv_block_size.MLX5DV_BLOCK_SIZE_512,
                                          dek=self.dek, initial_tweak=initial_tweak)
             self.client.qp.wr_set_mkey_crypto(crypto_attr)
             self.client.qp.wr_complete()
@@ -352,17 +344,17 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         """
         Configure an mkey with signature attributes.
         """
-        sig_crc = Mlx5SigCrc(crc_type=dve.MLX5DV_SIG_CRC_TYPE_CRC32, seed=0xFFFFFFFF)
-        sig_block_domain = Mlx5SigBlockDomain(sig_type=dve.MLX5DV_SIG_TYPE_CRC, crc=sig_crc,
-                                              block_size=dve.MLX5DV_BLOCK_SIZE_512)
+        sig_crc = Mlx5SigCrc(crc_type=mlx5dv_sig_crc_type.MLX5DV_SIG_CRC_TYPE_CRC32, seed=0xFFFFFFFF)
+        sig_block_domain = Mlx5SigBlockDomain(sig_type=mlx5dv_sig_type.MLX5DV_SIG_TYPE_CRC, crc=sig_crc,
+                                              block_size=mlx5dv_block_size.MLX5DV_BLOCK_SIZE_512)
         sig_attr = Mlx5SigBlockAttr(wire=sig_block_domain,
-                                    check_mask=dve.MLX5DV_SIG_MASK_CRC32)
+                                    check_mask=mlx5dv_sig_mask.MLX5DV_SIG_MASK_CRC32)
         self.client.qp.wr_set_mkey_sig_block(sig_attr)
 
     def get_send_wr(self, player, wire_encryption):
         mkey = player.wire_enc_mkey if wire_encryption else player.mem_enc_mkey
         sge = SGE(0, player.msg_size/2, mkey.lkey)
-        return SendWR(opcode=e.IBV_WR_SEND, num_sge=1, sg=[sge])
+        return SendWR(opcode=ibv_wr_opcode.IBV_WR_SEND, num_sge=1, sg=[sge])
 
     def get_recv_wr(self, player, wire_encryption):
         offset = 0 if wire_encryption else player.msg_size/2
@@ -417,10 +409,10 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         :param create_dek_func: Function that creates a DEK.
         """
         self.init_data_validation()
-        mkey_flags = dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_CRYPTO | \
-            dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT
+        mkey_flags = mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_CRYPTO | \
+            mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT
         self.create_players(Mlx5CryptoResources,
-                            dv_send_ops_flags=dve.MLX5DV_QP_EX_WITH_MKEY_CONFIGURE,
+                            dv_send_ops_flags=mlx5dv_qp_create_send_ops_flags.MLX5DV_QP_EX_WITH_MKEY_CONFIGURE,
                             mkey_create_flags=mkey_flags, msg_size=self.msg_size)
         create_dek_func()
         self.reg_client_mkey()
@@ -462,12 +454,12 @@ class Mlx5CryptoTrafficTest(Mlx5RDMATestCase):
         this mkey.
         """
         if 'wrapped_256_bits_key' in self.crypto_details:
-            self.key_size = dve.MLX5DV_CRYPTO_KEY_SIZE_256
-        mkey_flags = dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_CRYPTO | \
-                     dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT | \
-                     dve.MLX5DV_MKEY_INIT_ATTR_FLAGS_BLOCK_SIGNATURE
+            self.key_size = mlx5dv_crypto_key_size.MLX5DV_CRYPTO_KEY_SIZE_256
+        mkey_flags = mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_CRYPTO | \
+                     mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_INDIRECT | \
+                     mlx5dv_mkey_init_attr_flags.MLX5DV_MKEY_INIT_ATTR_FLAGS_BLOCK_SIGNATURE
         self.create_players(Mlx5CryptoResources,
-                            dv_send_ops_flags=dve.MLX5DV_QP_EX_WITH_MKEY_CONFIGURE,
+                            dv_send_ops_flags=mlx5dv_qp_create_send_ops_flags.MLX5DV_QP_EX_WITH_MKEY_CONFIGURE,
                             mkey_create_flags=mkey_flags)
         self.create_client_dek()
         self.reg_client_mkey(signature=True)
